library json_value;

import 'dart:convert';

///Json type enum
enum JsonType { isMap, isList, isString, isNum, isBool, isNull, unknown }

///Json object
class JsonValue {
  final dynamic _value;

  /// Initialize
  JsonValue(this._value);

  ///_value type
  JsonType get type {
    var t = JsonType.unknown;
    if (_value is Map) {
      t = JsonType.isMap;
    } else if (_value is List) {
      t = JsonType.isList;
    } else if (_value is String) {
      t = JsonType.isString;
    } else if (_value is num) {
      t = JsonType.isNum;
    } else if (_value is bool) {
      t = JsonType.isBool;
    } else if (_value == null) {
      t = JsonType.isNull;
    }
    return t;
  }

  ///key is int or String
  ///If key is int & type is Map ,return list[key] else _nullValue
  ///If key is String & type is List ,return map[key] else _nullValue
  JsonValue operator [](dynamic key) {
    if (key is int) {
      if (type == JsonType.isList) {
        return (key < listValue.length && key >= 0)
            ? listValue[key]
            : _nullValue;
      } else {
        return _nullValue;
      }
    } else if (key is String) {
      if (type == JsonType.isMap) {
        return mapValue[key] ?? _nullValue;
      } else {
        return _nullValue;
      }
    }
    return _nullValue;
  }

  /// If _value is Map ,return the map which value`s type is JsonValue
  Map<dynamic, JsonValue> get mapValue {
    Map<dynamic, JsonValue> map = {};
    if (type == JsonType.isMap) {
      (_value as Map<dynamic, dynamic>).forEach((key, value) {
        map[key] = JsonValue(value);
      });
    }
    return map;
  }

  /// If _value is Map ,return the map which value`s type is  original
  Map<dynamic, dynamic>? get mapObject {
    if (type == JsonType.isMap) {
      return _value as Map<dynamic, dynamic>;
    } else {
      return null;
    }
  }

  /// If _value is List ,return the list which element`s type is  JsonValue
  List<JsonValue> get listValue {
    List<JsonValue> list = [];
    if (type == JsonType.isList) {
      // (_value as List<dynamic>).
      (_value as List<dynamic>).forEach((element) {
        list.add(JsonValue(element));
      });
    }
    return list;
  }

  /// If _value is List ,return the list which element`s type is original
  List<dynamic>? get listObject {
    if (type == JsonType.isList) {
      return _value as List<dynamic>;
    } else {
      return null;
    }
  }

  /// Optional string
  String? get string {
    if (type == JsonType.isString) {
      return _value as String;
    } else {
      return null;
    }
  }

  /// string
  String get stringValue {
    if (type == JsonType.isString) {
      return _value as String;
    }
    if (type == JsonType.isNum) {
      return (_value as num).toString();
    }
    if (type == JsonType.isBool) {
      return (_value as bool) ? "true" : "false";
    } else {
      return "";
    }
  }

  ///Optional num
  num? get optionNum {
    if (type == JsonType.isNum) {
      return _value as num;
    } else {
      return null;
    }
  }

  ///num
  num get numValue {
    if (type == JsonType.isNum) {
      return _value as num;
    } else if (type == JsonType.isString) {
      return double.tryParse((_value as String)) ?? 0;
    } else if (type == JsonType.isBool) {
      return (_value as bool) ? 1 : 0;
    } else {
      return 0;
    }
  }

  ///Optional bool
  bool? get optionBool {
    if (type == JsonType.isBool) {
      return _value as bool;
    } else {
      return null;
    }
  }

  /// bool
  bool get booValue {
    if (type == JsonType.isBool) {
      return _value as bool;
    } else if (type == JsonType.isNum) {
      return (_value as num) == 0 ? false : true;
    } else if (type == JsonType.isString) {
      var s = stringValue.toLowerCase();
      bool v = false;
      if (s == "true" || s == "1") {
        v = true;
      } else if (s == "false" || s == "0") {
        v = false;
      }
      return v;
    } else {
      return false;
    }
  }

  /// object to string
  String toJsonString() {
    return json.encode(_value);
  }

  /// null value
  static final JsonValue _nullValue = JsonValue(null);
}
